package net.sf.corn.gate;

import java.lang.reflect.Method;

/**
 * Execution target is a wrapper class for service intance and {@link Method}, so the method 
 * can be executed by calling {@link Method#invoke} method with the desired parameters. 
 * The class also keeps service definition information  and operation definition information 
 * as meta data, so that the executers can take care of attached policies. Execution targets 
 * can be achieved by calling {@link Registry#getExecutionTarget} methods.
 * <p>
 * 	Since the class is designed for internal purposes, do not pay attention to it.
 * </p> 
 * @author Serhat Dirik
 */
class ExecutionTarget {

	private ServiceDefinition serviceDef;
	private OperationDefinition operationDef;
	private Object   serviceInstance;
	private Method   method;
	
	ExecutionTarget(ServiceDefinition serviceDef, OperationDefinition operationDef,
			Object serviceInstance, Method method) {
		super();
		this.serviceDef = serviceDef;
		this.operationDef = operationDef;
		this.serviceInstance = serviceInstance;
		this.method = method;
	}
	
	ServiceDefinition getServiceDef() {
		return serviceDef;
	}
	
	OperationDefinition getOperationDef() {
		return operationDef;
	}
	
	Object getServiceInstance() {
		return serviceInstance;
	}
	
	Method getMethod() {
		return method;
	}
	
	@Override
	public String toString() {
		return serviceDef.toString()+"."+operationDef.toString()+" ->"+method.toGenericString();
	}
}
