package net.sf.corn.gate;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

/**
 * Service proxy objects are gathered from the {@link CallDelegator#getServiceProxy}
 * method. This class instances created by call delegator to internal usage purposes.  
 * @author Serhat Dirik
 *
 */
public class ServiceProxy implements InvocationHandler {

	private ServiceDefinition serviceDef =null ;
	private IClientContext context = null;
	
	ServiceProxy(ServiceDefinition serviceDef , IClientContext context) {
		this.serviceDef = serviceDef;
		this.context = context;
	}

	/**
	 * Delegates a method execution to the call delegator
	 */
	@Override
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {
		Object retVal=null;
		switch(method.getName()){
			case	"accessClientContext":
				if(args == null || args.length == 0){
					retVal = this.context;
					break;
				}
			default:
				OperationDefinition op = null;
				for(OperationDefinition opDef : serviceDef.getOperations()){
					if(method.getName().equals(opDef.getMethodTarget())){
						op = opDef;
						break;
					}
				}
				if(op != null){
					ServiceRequest request = new ServiceRequest(serviceDef.getName(), op.getName(), args,context);
					ServiceResponse resp = CallDelegator.execute(request);
					if(resp.hasError()){
						if(resp.isExceptionTypeExpected()){
							throw resp.getReturnedException();
						}else{
							throw new GateException(resp.getReturnedException().getLocalizedMessage(),resp.getReturnedException()).code(GateException.CODE_SERVICE_UNEXPECTED);
						}
					}else{
						retVal = resp.getReturnedValue();
					}
				}else{
					//Never expect this
					throw new GateException(method.getName()+" method is not mapped to an service operation").code(GateException.CODE_REGISTRY_OPERATION_NOT_FOUND);
				}
		}
		return retVal;
	}

}
