package net.sf.corn.gate;

import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

/**
 * A simple implementation of {@link IDataStage} that  holds data in memory
 * @author Serhat Dirik
 */
public class SimpleConcurrentDataStage implements IDataStage {

	private static final long serialVersionUID = -8340376356547279619L;
	private String id;
	private Map<String,Object> platformAttributes = new ConcurrentHashMap<String, Object>();
	private Scope dataAreaScope = null; 
	
	public SimpleConcurrentDataStage(Scope dataAreaScope) {
		
		this.dataAreaScope = dataAreaScope;
		
	}
	
	public SimpleConcurrentDataStage(Scope dataAreaScope, String id) {
		if(dataAreaScope == null){
			throw new IllegalArgumentException("Data Area Scope Type cannot be null");
		}
		this.dataAreaScope = dataAreaScope;
		if(id == null){
			id = UUID.randomUUID().toString();
		}else{
			id = this.id;
		}
	}
	@Override
	public String getId() {
		return id;
	}

	@Override
	public Object getAttribute(String name) {
		return platformAttributes.get(name);
	}

	@Override
	public Iterable<String> getAttributeNames() {
		 
		return platformAttributes.keySet();
	}

	@Override
	public void removeAttribute(String name) {
		 platformAttributes.remove(name);

	}

	@Override
	public void setAttribute(String name, Object value) {
		 platformAttributes.put(name, value);
	}

	@Override
	public Scope getScope() {
		return dataAreaScope;
	}


	@Override
	public boolean hasAttribute(String name) {
		return platformAttributes.containsKey(name);
	}

}
